(* Taken from findlib.  Findlib is copyright 1999 by Gerd Stolpmann and
  distributed under the terms given in file LICENSE-findlib in this
  directory. *)

(*
  Load this module in the toplevel to install printers for the following types
  defined in the "num" library:
    - Nat.nat
    - Big_int.big_int
    - Ratio.ratio
    - Num.num

  No functions exported.

  Copyright (C) 2003  Stefano Zacchiroli <zack@debian.org>
*)

